@@ -1,31 +1,28 @@
-from __future__ import unicode_literals
-
 import random
 import string
-
 from datetime import timedelta
+
 from django import forms
 from django.conf import settings
 from django.contrib import messages, auth
-from django.contrib.auth import get_user_model, REDIRECT_FIELD_NAME
-from django.contrib.auth.forms import AuthenticationForm
+from django.contrib.auth import get_user_model
 from django.contrib.auth.models import Group
-from django.contrib.auth.views import login as django_login
+from django.contrib.auth.views import LoginView
 from django.core.mail import send_mail
-from django.core.urlresolvers import reverse
+from django.urls import reverse
 from django.http import HttpResponseRedirect
 from django.template import loader
 from django.utils import timezone
 from django.utils.translation import ugettext_lazy as _
 from django.views.generic import TemplateView
-from guardian.conf import settings as guardian_settings
+
 from smartmin.email import build_email_context
 from smartmin.views import SmartCRUDL, SmartView, SmartFormView, SmartListView, SmartCreateView, SmartUpdateView
 from .models import RecoveryToken, PasswordHistory, FailedLogin, is_password_complex
 
 
 class UserForm(forms.ModelForm):
-    new_password = forms.CharField(label=_("New Password"), widget=forms.PasswordInput)
+    new_password = forms.CharField(label=_("New Password"), widget=forms.PasswordInput, strip=False)
     groups = forms.ModelMultipleChoiceField(widget=forms.CheckboxSelectMultiple,
                                             queryset=Group.objects.all(), required=False)
 
@@ -66,7 +63,7 @@ class UserForm(forms.ModelForm):
 
 
 class UserUpdateForm(UserForm):
-    new_password = forms.CharField(label=_("New Password"), widget=forms.PasswordInput, required=False)
+    new_password = forms.CharField(label=_("New Password"), widget=forms.PasswordInput, required=False, strip=False)
 
     def clean_new_password(self):
         password = self.cleaned_data['new_password']
@@ -83,9 +80,11 @@ class UserUpdateForm(UserForm):
 
 
 class UserProfileForm(UserForm):
-    old_password = forms.CharField(label=_("Password"), widget=forms.PasswordInput, required=False)
-    new_password = forms.CharField(label=_("New Password"), widget=forms.PasswordInput, required=False)
-    confirm_new_password = forms.CharField(label=_("Confirm Password"), widget=forms.PasswordInput, required=False)
+    old_password = forms.CharField(label=_("Password"), widget=forms.PasswordInput, required=False, strip=False)
+    new_password = forms.CharField(label=_("New Password"), widget=forms.PasswordInput, required=False, strip=False)
+    confirm_new_password = forms.CharField(
+        label=_("Confirm Password"), widget=forms.PasswordInput, required=False, strip=False
+    )
 
     def clean_old_password(self):
         user = self.instance
@@ -132,12 +131,12 @@ class UserForgetForm(forms.Form):
 
 
 class SetPasswordForm(UserForm):
-    old_password = forms.CharField(label=_("Current Password"), widget=forms.PasswordInput, required=True,
+    old_password = forms.CharField(label=_("Current Password"), widget=forms.PasswordInput, required=True, strip=False,
                                    help_text=_("Your current password"))
     new_password = forms.CharField(label=_("New Password"), widget=forms.PasswordInput, required=True,
-                                   help_text=_("Your new password."))
+                                   help_text=_("Your new password."), strip=False)
     confirm_new_password = forms.CharField(label=_("Confirm new Password"), widget=forms.PasswordInput, required=True,
-                                           help_text=_("Confirm your new password."))
+                                           help_text=_("Confirm your new password."), strip=False)
 
     def clean_old_password(self):
         user = self.instance
@@ -200,11 +199,8 @@ class UserCRUDL(SmartCRUDL):
             if group_id:
                 queryset = queryset.filter(groups=group_id)
 
-            # Ignore superusers, staff users and anonymous users
-            return queryset.exclude(username=guardian_settings.ANONYMOUS_USER_NAME)\
-                .exclude(is_staff=True)\
-                .exclude(is_superuser=True)\
-                .exclude(password=None)
+            # ignore superusers and staff users
+            return queryset.exclude(is_staff=True).exclude(is_superuser=True).exclude(password=None)
 
         def get_name(self, obj):
             return obj.get_full_name()
@@ -340,7 +336,7 @@ class UserCRUDL(SmartCRUDL):
             return context_data
 
         def has_permission(self, request, *args, **kwargs):
-            return request.user.is_authenticated()
+            return request.user.is_authenticated
 
         def get_object(self, queryset=None):
             return self.request.user
@@ -361,7 +357,8 @@ class UserCRUDL(SmartCRUDL):
 
         def pre_process(self, request, *args, **kwargs):
             user = self.get_object()
-            login(request)
+
+            Login.as_view()(request)
 
             # After logging in it is important to change the user stored in the session
             # otherwise the user will remain the same
@@ -428,45 +425,52 @@ class UserCRUDL(SmartCRUDL):
             return context
 
 
-def login(request, template_name='smartmin/users/login.html',
-          redirect_field_name=REDIRECT_FIELD_NAME,
-          authentication_form=AuthenticationForm,
-          current_app=None, extra_context=None):
+class Login(LoginView):
+    template_name = 'smartmin/users/login.html'
 
-    lockout_timeout = getattr(settings, 'USER_LOCKOUT_TIMEOUT', 10)
-    failed_login_limit = getattr(settings, 'USER_FAILED_LOGIN_LIMIT', 5)
-    allow_email_recovery = getattr(settings, 'USER_ALLOW_EMAIL_RECOVERY', True)
+    def get_context_data(self, **kwargs):
+        context = super().get_context_data(**kwargs)
 
-    if request.method == "POST":
-        if 'username' in request.POST and 'password' in request.POST:
-            username = request.POST['username']
+        context['allow_email_recovery'] = getattr(settings, 'USER_ALLOW_EMAIL_RECOVERY', True)
 
-            user = get_user_model().objects.filter(username__iexact=username).first()
+        return context
 
-            # this could be a valid login by a user
-            if user:
+    def post(self, request, *args, **kwargs):
+        form = self.get_form()
 
-                # incorrect password?  create a failed login token
-                valid_password = user.check_password(request.POST['password'])
-                if not valid_password:
-                    FailedLogin.objects.create(user=user)
+        # clean form data
+        form_is_valid = form.is_valid()
 
-                bad_interval = timezone.now() - timedelta(minutes=lockout_timeout)
-                failures = FailedLogin.objects.filter(user=user)
+        lockout_timeout = getattr(settings, 'USER_LOCKOUT_TIMEOUT', 10)
+        failed_login_limit = getattr(settings, 'USER_FAILED_LOGIN_LIMIT', 5)
 
-                # if the failures reset after a period of time, then limit our query to that interval
-                if lockout_timeout > 0:
-                    failures = failures.filter(failed_on__gt=bad_interval)
+        user = get_user_model().objects.filter(username__iexact=form.cleaned_data.get('username')).first()
 
-                # if there are too many failed logins, take them to the failed page
-                if len(failures) >= failed_login_limit:
-                    return HttpResponseRedirect(reverse('users.user_failed'))
+        # this could be a valid login by a user
+        if user:
 
-                # delete failed logins if the password is valid
-                elif valid_password:
-                    FailedLogin.objects.filter(user=user).delete()
+            # incorrect password?  create a failed login token
+            valid_password = user.check_password(form.cleaned_data.get('password'))
+            if not valid_password:
+                FailedLogin.objects.create(user=user)
 
-    return django_login(request, template_name='smartmin/users/login.html',
-                        redirect_field_name=REDIRECT_FIELD_NAME,
-                        authentication_form=AuthenticationForm,
-                        current_app=None, extra_context=dict(allow_email_recovery=allow_email_recovery))
+            bad_interval = timezone.now() - timedelta(minutes=lockout_timeout)
+            failures = FailedLogin.objects.filter(user=user)
+
+            # if the failures reset after a period of time, then limit our query to that interval
+            if lockout_timeout > 0:
+                failures = failures.filter(failed_on__gt=bad_interval)
+
+            # if there are too many failed logins, take them to the failed page
+            if len(failures) >= failed_login_limit:
+                return HttpResponseRedirect(reverse('users.user_failed'))
+
+            # delete failed logins if the password is valid
+            elif valid_password:
+                FailedLogin.objects.filter(user=user).delete()
+
+        # pass through the normal login process
+        if form_is_valid:
+            return self.form_valid(form)
+        else:
+            return self.form_invalid(form)
